;;; ----------------------------------------------------------------------
;;;
;;; ILTBINDCPLIB: Generates a binding to the C++ standard library
;;;
;;; ----------------------------------------------------------------------
;;;
;;; This file is part of ILOG Talk.
;;; Inquiries to ILOG S.A.
;;;              9 rue de Verdun, BP 85,
;;;              94253 Gentilly Cedex, France.
;;;
;;; Copyright (c) 1994-1997 by ILOG
;;; ----------------------------------------------------------------------
;;;
;;; Call
;;;
;;;     (bind-cplib)
;;;
;;; This will generate in the $ILT_DIR/iltcplib hierarchy:
;;;
;;; - the libraries libiltcplibe, libiltcplibm and libiltcplibt
;;; - the workspace iltcplib.
;;;

(defun bind-cplib ()
  (.setup-environment)
  (.build-externals)
  (.build-binding)
  (.build-prunits)
  'done)

(defun .setup-environment ()
  ;; Setup the environment for the C++lib binding generation.
  (fprintf (stderr) "\nPreparing workspace\n")
  (fprintf (stderr) "-------------------\n")
  (unless (access #f"${ILT_DIR}/iltcplib/${PORTNAME}")
    (mkdir #f"${ILT_DIR}/iltcplib/${PORTNAME}")
    (mkdir #f"${ILT_DIR}/iltcplib/${PORTNAME}/o"))
  (unless (access #f"${ILT_DIR}/iltcplib/c")
    (mkdir #f"${ILT_DIR}/iltcplib/c"))
  (pushf (workspace-path) #f"${ILT_DIR}/iltcplib/src")
  (with-frozen-ws-cache
    (setf (wsref 'iltcplibs 'program-units:) ())
    (write-ws-file 'iltcplibs))
  (refresh-workspace 'iltcplibs)  
  (setf (default-workspace) 'iltcplibs)
  (in-workspaces '(talk)))

(defun .build-externals ()
  (fprintf (stderr) "\nCompiling externals\n")
  (fprintf (stderr) "-------------------\n")
  (let* ((os (internal.operating-system-family))
         (src-dir
          (case os
            (unix "$ILT_DIR/iltcplib/common")
            (win32 "%ILT_DIR%\\iltcplib\\common")))
         (dst-dir
          (case os
            (unix (catenate "$ILT_DIR/iltcplib/" (port-name) "/o"))
            (win32 (catenate "%ILT_DIR%\\iltcplib\\" (port-name) "\\o")))))
    (for ((e in: '(iltcplibrtecpp)))
      (fprintf (stderr) "Compiling %s\n" e)
      (abeille.invoke-llcpp e dst-dir src-dir 'standard ""))))

(defun .build-binding ()
  (bind-cpp-api '(#f"iltiostream.h")
                #f"${ILT_DIR}/iltcplib/src/iltcplib"
                cppparse-options: "-DILT_BINDING_TIME"
                cpp-options: .cpp-options
                link-options: .link-options
                buildp: ()
) )

(defun .build-prunits ()
  ;; Analyze and build the binding program units.
  (fprintf (stderr) "\nBuilding program units\n")
  (fprintf (stderr) "----------------------\n")
  (for ((p in: '(iltcplibt iltcplibtypt libiltcplibt)))
    (analyze-program-unit p buildp: t))
  (load-program-unit 'libiltcplibt)
  (for ((p in: '(iltcplibm iltcplibtypm libiltcplibm)))
    (analyze-program-unit p buildp: t))
  (for ((p in: '(iltcplibrte iltcplibe iltcplibtype libiltcplibe)))
    (analyze-program-unit p buildp: t))
  (refresh-workspace 'iltcplib)
)

(defun .changed-iltcplib-cpp-options ()
  ;; Call this when the .cpp-options have changed.
  (for ((m in: '(iltcplibe)))
    (with-frozen-td-cache
      (setf (tdref m 'cpp-options:) .cpp-options)
      (write-td-file m)
) ) )

(defun .changed-iltcplib-link-options ()
  ;; Call this when the .link-options have changed.
  (for ((m in: '(iltcplibe)))
    (with-frozen-td-cache
      (setf (tdref m 'link-options:) internal.link-options)
      (write-td-file m)
  ) )
  (analyze-program-unit 'libiltcplibe)
  (analyze-program-unit 'italkcplib)
)

(defglobal .cpp-options
  ;; Options to be passed to the C++ compiler.
  '((|sparc_4_2.1|   "-I${ILT_DIR}/iltcplib/common")
    (|sparc_4_3.0|   "-I${ILT_DIR}/iltcplib/common")
    (|sparc_4_4.0|   "-I${ILT_DIR}/iltcplib/common")
    (sparc_4_gnu     "-I${ILT_DIR}/iltcplib/common")
    (|sparc_5_3.0|   "-I${ILT_DIR}/iltcplib/common")
    (|sparc_5_4.0|   "-I${ILT_DIR}/iltcplib/common")
    (|sparc_5_4.1|   "-I${ILT_DIR}/iltcplib/common")
    (|sparc_5_4.2|   "-I${ILT_DIR}/iltcplib/common")
    (sparc_5_gnu     "-I${ILT_DIR}/iltcplib/common")
    (hp700           "-I${ILT_DIR}/iltcplib/common")
    (hp_10           "-I${ILT_DIR}/iltcplib/common")
    (|hp_10_1.0|     "-I${ILT_DIR}/iltcplib/common")
    (rs6000          "-I${ILT_DIR}/iltcplib/common")
    (|sgi_5.3_4|     "-I${ILT_DIR}/iltcplib/common")
    (|sgi32_6.2_6.2| "-I${ILT_DIR}/iltcplib/common")
    (|sgi32_6.2_7.0| "-I${ILT_DIR}/iltcplib/common")
    (|sgin32_6.2_7.0| "-I${ILT_DIR}/iltcplib/common")
    (|sgi64_6.2_7.0| "-I${ILT_DIR}/iltcplib/common")
    (alpha_3         "-I${ILT_DIR}/iltcplib/common")
    (|alpha_4_5.5|   "-I${ILT_DIR}/iltcplib/common")
    (|alpha_4_6.1|   "-I${ILT_DIR}/iltcplib/common")
    (|i86_linux1.2_gnu2.7| "-I${ILT_DIR}/iltcplib/common")
    (|i86_linux2.0_gnu2.7| "-I${ILT_DIR}/iltcplib/common")
    (|i86_linux2.0_egcs1.1| "-I${ILT_DIR}/iltcplib/common")
    (|sparc_linuxlibc6_egcs1.1| "-I${ILT_DIR}/iltcplib/common")
    (|i86_sunos5_3.0| "-I${ILT_DIR}/iltcplib/common")
    (|i86_sunos5_4.0| "-I${ILT_DIR}/iltcplib/common")
    (msvc4           "/I%ILT_DIR%\\iltcplib\\common /MD")
    (msvc5           "/I%ILT_DIR%\\iltcplib\\common /MD")
   )
)

(defglobal .link-options
  ;; Options to be passed to the linker.
  '((|sparc_4_2.1| "-lC")
    (|sparc_4_3.0| "-lC")
    (|sparc_4_4.0| "-lC")
    (|sparc_5_3.0| "/usr/lib/libC.so.3")
    (|sparc_5_4.0| "/usr/lib/libC.so.5")
    (|sparc_5_4.1| "/usr/lib/libC.so.5")
    (|sparc_5_4.2| "/usr/lib/libC.so.5")
    (|i86_sunos5_3.0| "/usr/lib/libC.so.3")
    (|i86_sunos5_4.0| "/usr/lib/libC.so.5")
    (hp700 "-lC")
    (hp_10 "-lC")
    (|hp_10_1.0| "-lCsup")
    (rs6000 "-L/usr/lpp/xlC/lib -lC")
    (|sgi_5.2_3| "-lC")
    (|sgi_5.3_4| "-lC")
    (|sgi32_6.2_6.2| "-lC")
    (|sgi32_6.2_7.0| "-lC")
    (|sgin32_6.2_7.0| "-lC")
    (|sgi64_6.2_7.0| "-lC")
    (alpha_3 "")
    (|alpha_4_5.5| "")
    (|alpha_4_6.1| ""))
)
